package com.lgfischer.pageflipdemo;

import android.content.Context;
import android.util.Log;
import android.view.MotionEvent;
import android.view.VelocityTracker;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.ViewGroup;
import android.widget.Scroller;

/**
 * Implementation of a page-view like for Android. Add clickable childs on it
 * and each child will be shown as a page. For more info, please go to
 * http://coderender.blogspot.com/2011/10/android-sdk-using-scroller-object-to.html
 */
public class PageFlipper extends ViewGroup {

	private static final String TAG = "PageViewer";

	private static final int ST_IDLE = 'I';
	private static final int ST_WATCHING = 'W';
	private static final int ST_DRAGGING = 'D';
	private static final int ST_ANIMATING = 'A';
	private static final int ST_IGNORING = 'G';

	private int mState = ST_IDLE;

	private int mFirstX = -1;
	private int mFirstY = -1;
	private int mLastX = -1;

	private Scroller mScroller;
	private VelocityTracker mVTracker;

	private int mTouchSlop;
	private int mFlingSlop;

	private void setState(int newState) {
		Log.i(TAG, "State transition from " + (char) getState() + " to "
				+ (char) newState);
		mState = newState;
	}

	private int getState() {
		return mState;
	}

	public PageFlipper(Context context) {
		super(context);
		final ViewConfiguration configuration = ViewConfiguration
				.get(getContext());
		mTouchSlop = configuration.getScaledTouchSlop();
		mFlingSlop = configuration.getScaledMinimumFlingVelocity();

		mScroller = new Scroller(context);
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {

		setMeasuredDimension(getDefaultSize(0, widthMeasureSpec),
				getDefaultSize(0, heightMeasureSpec));

		int childWidthMeasureSpec;
		int childHeightMeasureSpec;

		// Children are just made to fill our space.
		childWidthMeasureSpec = MeasureSpec.makeMeasureSpec(getMeasuredWidth()
				- getPaddingLeft() - getPaddingRight(), MeasureSpec.EXACTLY);
		childHeightMeasureSpec = MeasureSpec.makeMeasureSpec(
				getMeasuredHeight() - getPaddingTop() - getPaddingBottom(),
				MeasureSpec.EXACTLY);

		final int size = getChildCount();
		for (int i = 0; i < size; ++i) {
			final View child = getChildAt(i);
			if (child.getVisibility() != GONE) {
				child.measure(childWidthMeasureSpec, childHeightMeasureSpec);
			}
		}
	}

	@Override
	protected void onLayout(boolean changed, int left, int top, int right,
			int bottom) {
		int width = right - left;

		final int childCount = getChildCount();
		for (int i = 0; i < childCount; ++i) {
			final View v = getChildAt(i);
			v.layout(left, top, right, bottom);
			left += width;
			right += width;
		}
	}

	@Override
	public boolean onInterceptTouchEvent(MotionEvent event) {
		Log.i(TAG, "onInterceptTouchEvent page viewer: state="
				+ (char) getState());

		int action = event.getAction();

		if (action == MotionEvent.ACTION_DOWN && getState() == ST_IDLE) {
			if (mVTracker != null) {
				mVTracker.recycle();
			}
			mVTracker = VelocityTracker.obtain();
			mVTracker.addMovement(event);
			setState(ST_WATCHING);
			mLastX = mFirstX = (int) event.getX();
			mFirstY = (int) event.getY();
			Log.i(TAG, "First=" + mFirstX);
			return false;
		}
		if (action == MotionEvent.ACTION_MOVE && getState() == ST_WATCHING) {
			int deltaX = Math.abs(mFirstX - (int) event.getX());
			int deltaY = Math.abs(mFirstY - (int) event.getY());

			if (deltaX > mTouchSlop && deltaY < mTouchSlop) {
				setState(ST_DRAGGING);
				return true;
			}
			if (deltaX < mTouchSlop && deltaY > mTouchSlop) {
				setState(ST_IGNORING);
				return false;
			}
		}

		if (action == MotionEvent.ACTION_UP
				|| action == MotionEvent.ACTION_CANCEL) {
			setState(ST_IDLE);
			mVTracker.recycle();
			mVTracker = null;
		}
		return false;
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		Log.i(TAG, "onTouch page viewer: state=" + (char) getState());

		int action = event.getAction();

		if (action == MotionEvent.ACTION_DOWN) {
			if (mVTracker != null) {
				mVTracker.recycle();
				mVTracker = null;
			}
			mVTracker = VelocityTracker.obtain();
			mVTracker.addMovement(event);
		} else if (action == MotionEvent.ACTION_MOVE) {
			mVTracker.addMovement(event);
		}

		if (getState() == ST_WATCHING)
			return super.onTouchEvent(event);

		if (getState() == ST_DRAGGING && action == MotionEvent.ACTION_MOVE) {
			int deltaX = mLastX - (int) event.getX();
			scrollBy(deltaX, 0);
			mLastX = (int) event.getX();
		}
		if (getState() == ST_DRAGGING
				&& (action == MotionEvent.ACTION_UP || action == MotionEvent.ACTION_CANCEL)) {
			setState(ST_ANIMATING);

			mVTracker.computeCurrentVelocity(1000);
			float velocity = mVTracker.getXVelocity();
			final boolean fling = Math.abs(velocity) > mFlingSlop;

			final int width = getWidth();
			final int delta = mLastX - mFirstX;
			final boolean moveToNextScreen = Math.abs(delta) > (width * 0.5);

			if (fling || moveToNextScreen) {
				int motion = (delta > 0 ? -1 : 1) * (width - Math.abs(delta));
				mScroller.startScroll(getScrollX(), getScrollY(), motion, 0);
			} else {
				mScroller.startScroll(getScrollX(), getScrollY(), delta, 0);
			}

			invalidate();
			mLastX = mFirstX = mFirstY = -1;
			mVTracker.recycle();
			mVTracker = null;
		}
		return false;
	}

	@Override
	public void computeScroll() {
		if (mScroller.computeScrollOffset()) {
			scrollTo(mScroller.getCurrX(), mScroller.getCurrY());
			invalidate();
		} else {
			if (getState() == ST_ANIMATING && mScroller.isFinished())
				setState(ST_IDLE);
			super.computeScroll();
		}
	}
}
